#!/usr/bin/env python
#
# @todo: es ist vermutlich besser, nicht nur die Attachments,
#        sondern auch das Array last_seen 
#
import time
import datetime
from faker import Factory
from argparse import ArgumentParser
from flask import Flask, render_template, request, redirect
from elasticsearch import Elasticsearch
from flask_elasticsearch import FlaskElasticsearch

### setup
app = Flask(__name__)
es = FlaskElasticsearch(app)
fake = Factory.create('de_DE')
app.config['ES_INDEX'] = 'serialz'
app.config['ES_DOCTYPE'] = 'serialz'

@app.context_processor # @todo: not needed anymore
def inject_Elasticsearch(): return dict(es=es)


### some little helpers
def essearch(**kwargs):
    return es.search(index=app.config['ES_INDEX'],
                     doc_type=app.config['ES_DOCTYPE'],
                     size=10000,
                     **kwargs)
def esget(**kwargs):
    return es.get(index=app.config['ES_INDEX'],
                  doc_type=app.config['ES_DOCTYPE'],
                  **kwargs)
def esdelete(**kwargs):
    return es.delete(index=app.config['ES_INDEX'],
                     doc_type=app.config['ES_DOCTYPE'],
                     **kwargs)
def esindex(**kwargs):
    return es.index(index=app.config['ES_INDEX'],
                     doc_type=app.config['ES_DOCTYPE'],
                     **kwargs)


### functions mapped to URLs
@app.route('/')
def index():
    """start page with overview"""
    result = essearch(body={'query': {'match_all': {}}})
    return render_template('index.html', result=result, search='')


@app.route('/search', methods=['POST'])
def search():
    """simple text search
    @todo: implement enhanced search
    """
    result = essearch(body={'query': {'match_phrase': {'_all': request.form['search']}}})
    return render_template('index.html', result=result, search=request.form['search'])


@app.route('/device/<id>')
def device(id):
    """get one device by serial number"""
    return render_template('device.html', result=esget(id=id))
    

@app.route('/new_device', methods=['GET', 'POST'])
def new_device():
    """create a new device
    @todo: check if the serial already exists!
    @todo: validate input!
    """
    if request.method == 'GET':
        return render_template('new_device.html',
                               serial=fake.isbn13(), built=datetime.datetime.now().date().isoformat())
    elif request.method == 'POST':
        newdoc = {
            'serial': request.form['serial'],
            'buyer': request.form['buyer'],
            'built': request.form['built'],
            'last_seen': []
        }
        result = esindex(body=newdoc)
        print(result)
        time.sleep(1.0)
        return redirect('/device/%s'%(result['_id']))
        #return redirect('/')


@app.route('/delete_device/<id>', methods=['GET', 'POST'])
def delete_device(id):
    """deletes a device"""
    if request.method == 'POST':
        result = esdelete(id=id)
        time.sleep(1.0)
        return redirect('/')
    return render_template('confirm_delete_device.html', result=esget(id=id))


@app.route('/add_repair/<id>', methods=['GET', 'POST'])
def add_repair(id):
    if request.method == 'GET':
        return render_template('add_repair.html', result=esget(id=id))
    elif request.method == 'POST':
        subdoc = {
            'date': request.form['date'],
            'from': request.form['from'],
            'problem': request.form['problem'],
            'done': request.form['done'],
        }
        newdoc = esget(id=id)['_source']
        newdoc['last_seen'].append(subdoc)
        result = esindex(id=id, body=newdoc)
        return redirect('/device/%s'%(id))


#@app.route('/edit/<id>', methods=['GET', 'POST'])
#def edit(id):
#    pass
    

### "main" program
if __name__ == '__main__':
    parser = ArgumentParser(description='simple serial management')
    parser.add_argument('--debug', '-d', action='store_true', help='turn on debugging')
    parser.add_argument('--bind', '-b', type=str, default='localhost', help='ip address to bind to (default localhost)')
    parser.add_argument('--port', '-p', type=int, default='8080', help='port to bind to')
    parser.add_argument('--esurl', '-e', type=str, default='localhost:9200', help='elasticsearch base URL')
    parser.add_argument('--esauth', '-a', type=str, default=None, help='elasticsearch HTTP auth credentials')
    args = parser.parse_args()
    
    app.config['ELASTICSEARCH_HOST'] = args.esurl
    if args.esauth: app.config['ELASTICSEARCH_HTTP_AUTH'] = args.esauth
    app.run(debug=args.debug, host=args.bind, port=args.port)
